// --------------------------------------------------------------------------------------------------------------------
// <copyright file="DependencyInjectionHostFactory.cs" company="Witcraft LLC">
//   COPYRIGHT TEXT
// </copyright>
// <summary>
//   service 1 host.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

using System;
using System.ServiceModel;
using System.ServiceModel.Activation;

namespace WCFFacility
{
    /// <summary>
    /// Produces <see cref="DependencyInjectionHost"/>s.
    /// </summary>
    public class DependencyInjectionHostFactory : ServiceHostFactory
    {
        /// <summary>
        /// Creates a <see cref="DependencyInjectionHost"/> for a specified type of service with a specific base address. 
        /// </summary>
        /// <returns>
        /// A <see cref="DependencyInjectionHost"/> for the type of service specified with a specific base address.
        /// </returns>
        /// <param name="serviceType">
        /// Specifies the type of service to host. 
        /// </param>
        /// <param name="baseAddresses">
        /// The <see cref="T:System.Array"/> of type <see cref="T:System.Uri"/> that contains the base addresses for the service hosted.
        /// </param>
        protected override ServiceHost CreateServiceHost(Type serviceType, Uri[] baseAddresses)
        {
            return new DependencyInjectionHost(serviceType, baseAddresses);
        }
    }
}